#!/bin/bash
#
# created:     2008-05-19 (Christoph Sandhaus)
#
# modified: 2008-05-21 (Christoph Sandhaus)
# - testing for duplicate images before creating anim
# - moving duplicates (latest) to backup dir
#
# modified: 2008-05-25 (Christoph Sandhaus)
# - add date watermark to each image on top left
#

# EXECUTE has to be 0 to execute file operations
# EXECUTE not equal to 0 is used for testing
EXECUTE=0

# move duplicate images to backup dir
BACKUP_DIR="backup"

# directory for watermarks (date)
WM_DIR="wm"

# file extension for images
F_EXT=".png"

# prefix for watermark images
WM_PRE="wm_"


###
###
###
function doExecute()
{
	CMD="$1"

	[ $EXECUTE -eq 0 ] \
                && $CMD \
                || prot "FILE_OP: $CMD"

}


###
### create protocol output
###
function prot()
{
	D=$(date +'%Y/%m/%d,%H:%M:%S')
	echo "[$D,$$] $1"
}


###
### search for duplicate images
### move duplicates to backup dir BACKUP_DIR
###
function checkDuplicates()
{
	# the second of duplicate is removed if
	# - they are next by next in sorted list
	# - they are binary identical (diff)

	# dir to check
	IMG_DIR="$1"

	# create sorted file list array, exclude watermark files
	F_LIST=($(ls -1 "$IMG_DIR/"[!$WM_PRE]*${F_EXT}| sort))

	# file count
	CNT=${#F_LIST[@]}
	prot "checking $CNT files for duplicates in $1"

	# minimum of two files is needed
	[ $CNT -lt 2 ] && { prot "at least two files are needed, returning ..."; return; }

	# create $BACKUP_DIR dir
	doExecute "mkdir -p $IMG_DIR/$BACKUP_DIR"

	CUR=0                 # lower array id
	NEXT=$(expr $CUR + 1) # higher array id

	# endless loop
	while :
	do
		diff -s ${F_LIST[$CUR]} ${F_LIST[$NEXT]}>/dev/null
		IDENTICAL=$?
		if [ 0 -eq $IDENTICAL ]
		then
			F1=$(basename ${F_LIST[$CUR]})
			F2=$(basename ${F_LIST[$NEXT]})
			prot "$F1 ... $F2 are identical, moving $F2 to $BACKUP_DIR"
			doExecute "mv ${F_LIST[$NEXT]} $IMG_DIR/$BACKUP_DIR/"
			NEXT=$(expr $NEXT + 1)
		else
			prot "$(basename ${F_LIST[$CUR]}) ... $(basename ${F_LIST[$NEXT]}) differ"
			CUR=$(expr $CUR + 1)
			NEXT=$(expr $NEXT + 1)
		fi
		[ $NEXT -ge $CNT ] && break;
	done
}


function createWatermark()
{
	# dir do create watermarks
	IMG_DIR="$1"

	# create sorted file list array
        F_LIST=($(ls -1 "$IMG_DIR/"[!$WM_PRE]*${F_EXT}| sort))

	prot "creating watermarks for dir $IMG_DIR"
	for IMAGE in ${F_LIST[*]}
	do
		prot "creating date watermark for $IMAGE"
		F_IMAGE=$(basename $IMAGE)
		YEAR=${F_IMAGE:0:4}
		MON=${F_IMAGE:4:2}
		DAY=${F_IMAGE:6:2}
		[ $EXECUTE -eq 0 ] \
                	&& convert -fill black -draw "text 50,50 \
				'$YEAR-$MON-$DAY'" \
				$IMAGE $IMG_DIR/$WM_PRE$F_IMAGE \
                	|| prot "FILE_OP: convert -fill black -draw \
				\"text 50,50 '$YEAR-$MON-$DAY'\" \
				$IMAGE $IMG_DIR/$WM_PRE$F_IMAGE"

	done
}


###
### main
###
[ $EXECUTE -ne 0 ] && prot "===== TEST MODE: file operations are not executed!! ====="

if [ -z "$2" ]
then
	echo
	echo "$(basename $0) joins a collection of ${F_EXT}s to a single animated gif"
	echo
	echo "usage: $(basename $0) delay path"
	echo
	echo "delay: delay between single images (ms)"
	echo "path: path containing ${F_EXT}s to join to animated gif"
	echo
	echo "\"path\" is searched recoursively for ${F_EXT}. if a subdirectory"
	echo "contains more than one ${F_EXT}, these files are concatenated to"
	echo "a single file called \"anim.gif\""
	echo
	echo "example:"
	echo "$(basename $0) 200 /home/foo/Images"
	echo
fi

DELAY=$1
PNG_DIR=$2

prot "parameters: delay=$DELAY, path=$PNG_DIR"
LOOP_CNT=0

find $PNG_DIR -type f -name "*${F_EXT}" -printf "%h\n" | \
	sort | \
	uniq -c | \
	while read RECENT_DIR
	do
		LOOP_CNT=$(expr $LOOP_CNT + 1)
		VALUES=($(echo $RECENT_DIR))
		prot "loop $LOOP_CNT, values: ${VALUES[*]}"
		# content of VALUES:
		#   VALUES[0]: count of images
		#   VALUES[1]: path to images

		# ignore, if end of path is "$BACKUP_DIR"
		[ x"$(basename ${VALUES[1]})" = x"$BACKUP_DIR" ] && { prot "skipping $BACKUP_DIR ..."; continue; }

		# remove duplicate images
		checkDuplicates	$(pwd)/${VALUES[1]}

		# insert date into images
		createWatermark $(pwd)/${VALUES[1]}

		# create anim
		prot "creating anim for ${VALUES[1]} ... "
		doExecute "convert -delay $DELAY ${VALUES[1]}/${WM_PRE}*${F_EXT} ${VALUES[1]}/anim.gif"

		# remove watermark files
		doExecute "rm -f ${VALUES[1]}/${WM_PRE}*"
	done

